<?php

namespace Vanderbilt\REDCap\Classes\Fhir\FhirLauncher\Middlewares;

use Exception;
use Vanderbilt\REDCap\Classes\Fhir\FhirLauncher\FhirLauncher;
use Vanderbilt\REDCap\Classes\Fhir\FhirLauncher\States\State;

/**
 * Middleware for user authentication.
 */
class AuthenticationMiddleware implements MiddlewareInterface {
    /**
     * Handles user authentication middleware logic.
     *
     * This method checks if the user is authenticated and redirects to the login page if not.
     *
     * @param FhirLauncher $launcher The FHIR launcher instance.
     * @param State $state The state to process.
     * @return State Returns the processed state.
     */
    public static function handle($launcher, $state) {
         // Retrieve the current session from the launcher
        $session = $launcher->getSession();
        
        // Get the username from the session, if available
        $username = $_SESSION['username'] ?? false;

        // No username is set, call the login function to use the standard REDCap login form
        if ($username === false) loginFunction();
        
        // Check if a username is set and matches the session user
        if ($username === $session->user) {
            // Username matches, return the state
            return $state;
        } else {
            // Username is set but does not match the session user, send a 401 error
            throw new Exception('Unauthorized access: You do not have permission to view the requested page', 401);
        }
    }

}