<?php
namespace ExternalModules;

$project_id = $arguments[0];

?>
<script type="text/javascript">
	if(window.ExternalModules === undefined){
		window.ExternalModules = {}
	}

	window.ExternalModules.moduleDependentRequest = function(url, action){
		$.get(url, function(response){
			try{
				response = JSON.parse(response)
				action(response)
			}
			catch(e){
				if(response.startsWith('<!DOCTYPE')){
					console.error('An unexpected response was returned from ' + url)
					/**
					 * Assume the login page is being returned per:
					 * https://community.projectredcap.org/questions/138578
					 */
					return
				}

				// Escape the response to prevent injection
				response = $('<div/>').text(response).html()
				
				simpleDialog(
					"<p><?=ExternalModules::tt('em_errors_164')?>:</p><pre style='max-height: 50vh; white-space: pre-wrap;'>" + response + "</pre>",
					<?=json_encode(ExternalModules::tt('em_manage_75'))?>,
					'module-error-dialog',
					1000
				)
			}
		})
	}

	/**
	 * We queue ajax requests because concurrent requests for the same browser tab
	 * sometimes fail because they trigger REDCap's duplicate query protection.
	 * See this issue for details: https://github.com/vanderbilt-redcap/external-module-framework/issues/619
	 */
	window.ExternalModules.__ajaxQueue = (function() {
		let queue = Promise.resolve();
		// Return the "enqueue" function that will add a new request to the queue
		return (requestFunc) => {
			const task = queue.then(requestFunc);
			queue = task.catch(function() {
				/**
				 * Ensure errors don't interrupt the queue.
				 * This error will already be written to the console via a previous catch() call.
				 */
			}); 
			return task;
		};
	})();

	/**
	 * Processes a module ajax request, JSMO.ajax(), using a queue
	 */
	window.ExternalModules.moduleQueuedAjax = function(ajaxSettings, action, payload) {
		//#region Helpers
		function formatModuleName() {
			return ajaxSettings.prefix + '_' + ajaxSettings.version;
		}
		function parseResponse(response) {
			/**
			 * We explode and join here so that the LOGIN_PAGE_EXCERPT is not directly included in this javascript code,
			 * or the following response.includes(loginPageExcerpt) call will incorrectly match itself in some cases.
			 * Set ajaxSettings.csrfToken to an invalid value to test this case.
			 */
			const loginPageExcerpt = <?=json_encode(explode("'", Framework::LOGIN_PAGE_EXCERPT))?>.join("'")

			try{
				return JSON.parse(response);
			}
			catch(error){
				if(
					response.startsWith(<?=json_encode(Framework::REDCAP_PAGE_PREFIX)?>)
					&&
					response.includes(loginPageExcerpt)
				){
					error = <?=json_encode(ExternalModules::tt('em_errors_171'))?>;
				}
				throw error;
			}
		}
		function handleError(reject, error) {
			if (action == <?=json_encode(ExternalModules::MODULE_AJAX_LOGGING_ACTION)?>) {
				console.error('Module LOG request error for '+formatModuleName()+' - Error:', error, ', Message:', payload.msg, ', Parameters:', payload.params);
			}
			else {
				console.error('Module AJAX request error for '+formatModuleName()+' - Error:', error, ', Action:', action, ', Payload:', payload);
			}
			reject(error);
		}
		function packageData() {
			const formData = new FormData();
			formData.append('verification', ajaxSettings.verification);
			formData.append('action', action);
			formData.append('payload', JSON.stringify(payload));
			// This is required because REDCap core erases 'redcap_csrf_token' on __passthru requests.
			formData.append('redcap_external_module_csrf_token', ajaxSettings.csrfToken);
			// This is required for non-__passthru ajax requests.
			formData.append('redcap_csrf_token', ajaxSettings.csrfToken);
			return formData;
		}
		function makeRequest() {
			return new Promise(function(resolve, reject) {
				fetch(ajaxSettings.endpoint, {
					method: 'POST',
					body: packageData()
				})
				.then(response => response.text())
				.then(response => {
					response = parseResponse(response);
					if (response['success'] === true) {
						ajaxSettings.verification = response['verification'];
						resolve(response.payload ?? null);
					}
					else {
						handleError(reject, response.error);
					}
				})
				.catch(err => {
					handleError(reject, err);
				});
			});
		}
		//#endregion
		
		// Add the request to the queue and return the promise
		return ExternalModules.__ajaxQueue(makeRequest);
	};
</script>
<?php

if(in_array(ExternalModules::getUsername(), [null, \System::SURVEY_RESPONDENT_USERID])){
	// Skip the following code for unauthenticated users.
	return;
}

/**
 * This line shouldn't be here, but it has been for a long time,
 * and removing it might have downstream effects for modules.
 * I guess we'll leave it for now.
 */
set_include_path('.' . PATH_SEPARATOR . get_include_path());

$links = ExternalModules::getLinks();

$linkDisplayed = false;

?>
<script type="text/javascript">
	<?php
	/**
	 * This if statement prevents the redcap_module_link_check_display() call below
	 * from executing on the EM control center page, in order to allow admins to disable modules
	 * even there's something wrong preventing the module from being instantiated.
	 */
	if(!ExternalModules::isControlCenterPage()){ ?>
		$(function () {
			if ($('#project-menu-logo').length > 0) {
				var menubox = $('#external_modules_panel .x-panel-body .menubox .menubox')

				<?php
				foreach($links as $_=>$link){
					$prefix = $link['prefix'];
					$framework_instance = ExternalModules::getFrameworkInstance($prefix);
					$module_instance = $framework_instance->getModuleInstance();

					try{
						$new_link = $module_instance->redcap_module_link_check_display($project_id, $link);
						if($new_link){
							if(is_array($new_link)){
								$link = $new_link;
							}
							// Moved this check here, as it makes no sense to append a link that has no display name
							// (which could be the case after returning from the hook).
							if(empty($link["name"])){
								continue;
							}

							$linkDisplayed = true;
							?>
							menubox.append(<?=json_encode($framework_instance->getLinkIconHtml($link))?>);
							<?php
						}
					}
					catch(\Throwable $e){
						ExternalModules::sendAdminEmail(
							//= An exception was thrown when generating links
							ExternalModules::tt("em_errors_77"),
							$e->__toString(), $prefix);
					}
				}

				?>
			}
		})
	<?php } ?>
</script>

<?php

if(!$linkDisplayed && empty(ExternalModules::getMenuHeaderLinks($project_id))){
	/**
	 * Hide the External Modules section, since it doesn't contain any links.
	 * Hiding via style is faster than javascript, and prevents lag/flash for the user.
	 */
	echo "
		<style>
			#external_modules_panel{
				display: none;
			}
		</style>
	";
}

if(ExternalModules::isRoute('DataImportController:index')){
	ExternalModules::callHook('redcap_module_import_page_top', [$project_id]);
}
