<?php namespace ExternalModules\Sniffs\Misc;

use ExternalModules\SniffMessages;
use PHP_CodeSniffer\Files\File;
use PHP_CodeSniffer\Sniffs\Sniff;

class DataTableSniff implements Sniff{

    function register()
    {
        return [
            T_CONSTANT_ENCAPSED_STRING, // strings without variables
            T_DOUBLE_QUOTED_STRING, // strings containing variables
        ];
    }

    function process(File $file, $position)
    {
        $testsDir = realpath(__DIR__ . '/../../');
        if(strpos(realpath($file->path), $testsDir) === 0){
            // Ignore redcap_data references in tests.
            return;
        }
        
        $content = $file->getTokens()[$position]['content'];
        if(
            SniffMessages::containsDataTableName($content)
            &&
            !in_array($this->getLast($file, $position, T_STRING), ['getDataTable', 'project_id', 'logEvent', 'preg_replace'])
        ){
            $file->addWarning("A 'redcap_data' table name is hardcoded, which will prevent your module from finding data on newer projects.  Please add the following method to your module and use it instead of hardcoding 'redcap_data' in queries.  If your module's 'redcap-version-min' can be set to at least 14.0.0, you do not need to add this method as it will already be available:\n\n" .
            "    public function getDataTable(\$pid) {\n" .
            "        return method_exists('\REDCap', 'getDataTable') ?\n" .
            "            \REDCap::getDataTable(\$pid) : 'redcap_data';\n" .
            "    }\n\n", $position, 'Found');
        }
    }

    private function getLast($file, $position, $code){
        while($position > 0){
            $position--;
            $token = $file->getTokens()[$position];
            if($token['code'] === $code){
                return $token['content'];
            }
        }

        return null;
    }
}
